import sys
import yaml

import django
from django.conf import settings as django_settings
from djangoldp.conf.ldpsettings import LDPSettings
from djangoldp.tests.settings_default import yaml_config

# load test config
config = yaml.safe_load(yaml_config)
ldpsettings = LDPSettings(config)
django_settings.configure(ldpsettings)

django.setup()
from django.test.runner import DiscoverRunner

test_runner = DiscoverRunner(verbosity=1)

failures = test_runner.run_tests([
    'djangoldp.tests.tests_settings',
    'djangoldp.tests.tests_ldp_model',
    'djangoldp.tests.tests_ldp_viewset',
    'djangoldp.tests.tests_save',
    'djangoldp.tests.tests_user_permissions',
    'djangoldp.tests.tests_guardian',
    'djangoldp.tests.tests_anonymous_permissions',
    'djangoldp.tests.tests_update',
    'djangoldp.tests.tests_auto_author',
    'djangoldp.tests.tests_get',
    'djangoldp.tests.tests_delete',
    'djangoldp.tests.tests_sources',
    'djangoldp.tests.tests_pagination',
    'djangoldp.tests.tests_inbox',
    'djangoldp.tests.tests_backlinks_service',
    'djangoldp.tests.tests_cache'
])
if failures:
    sys.exit(failures)



"""
This module is meant to be used as a testing LDP package.

It contains configuration elements imported by djangoldp
when the django server is setup.
"""

# define an extra variables
MYPACKAGEVAR = 'ok'
USE_I18N = False

# register an extra middleware
MIDDLEWARE = [
    'djangoldp.tests.dummy.middleware.DummyMiddleware'
]

# register an extra installed app
INSTALLED_APPS = [
    'djangoldp.tests.dummy.apps.DummyConfig'
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-04-29 13:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0006_activity'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='allow_create_backlink',
            field=models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save'),
        ),
        migrations.AddField(
            model_name='ldpsource',
            name='allow_create_backlink',
            field=models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission, Group
from rest_framework.test import APIClient, APITestCase
from .models import JobOffer, LDPDummy, PermissionlessDummy

import json


class TestUserPermissions(APITestCase):

    def setUp(self):
        self.user = get_user_model().objects.create_user(username='john', email='jlennon@beatles.com', password='glass onion')
        self.client = APIClient(enforce_csrf_checks=True)
        self.client.force_authenticate(user=self.user)
        self.job = JobOffer.objects.create(title="job", slug="slug1")

    def setUpGroup(self):
        self.group = Group.objects.create(name='Test')
        view_perm = Permission.objects.get(codename='view_permissionlessdummy')
        self.group.permissions.add(view_perm)
        self.group.save()

    # list - simple
    def test_get_for_authenticated_user(self):
        response = self.client.get('/job-offers/')
        self.assertEqual(response.status_code, 200)

    # list - I do not have permission from the model, but I do have permission via a Group I am assigned
    # TODO: https://git.startinblox.com/djangoldp-packages/djangoldp/issues/291
    '''def test_group_list_access(self):
        self.setUpGroup()

        response = self.client.get('/permissionless-dummys/')
        self.assertEqual(response.status_code, 403)

        self.user.groups.add(self.group)
        self.user.save()
        response = self.client.get('/permissionless-dummys/')
        self.assertEqual(response.status_code, 200)'''

    # TODO: repeat of the above test on nested field
    '''def test_group_list_access_nested(self):
        self.setUpGroup()
        parent = LDPDummy.objects.create()
        dummy = PermissionlessDummy.objects.create(parent=parent)'''

    def test_get_1_for_authenticated_user(self):
        response = self.client.get('/job-offers/{}/'.format(self.job.slug))
        self.assertEqual(response.status_code, 200)

    def test_post_request_for_authenticated_user(self):
        post = {'title': "job_created", "slug": 'slug1'}
        response = self.client.post('/job-offers/', data=json.dumps(post), content_type='application/ld+json')
        self.assertEqual(response.status_code, 201)

    def test_put_request_for_authenticated_user(self):
        body = {'title':"job_updated"}
        response = self.client.put('/job-offers/{}/'.format(self.job.slug), data=json.dumps(body),
                                   content_type='application/ld+json')
        self.assertEqual(response.status_code, 200)

    def test_request_patch_for_authenticated_user(self):
        response = self.client.patch('/job-offers/' + str(self.job.slug) + "/",
                                   content_type='application/ld+json')
        self.assertEqual(response.status_code, 200)



import argparse
from django.apps import apps
from django.core.management.base import BaseCommand
from django.conf import settings
from djangoldp.models import LDPSource
from urllib.parse import urlparse
import requests

def isstring(target):
  if isinstance(target, str):
    return target
  return False

class Command(BaseCommand):
  help = "Check the datas integrity"

  def add_arguments(self, parser):
    parser.add_argument(
      "--ignore",
      action="store",
      default=False,
      type=isstring,
      help="Ignore any server, comma separated",
    )
    parser.add_argument(
      "--ignore-faulted",
      default=False,
      nargs="?",
      const=True,
      help="Ignore eventual faulted",
    )
    parser.add_argument(
      "--fix-faulted-resources",
      default=False,
      nargs="?",
      const=True,
      help="Fix faulted resources",
    )
    parser.add_argument(
      "--ignore-404",
      default=False,
      nargs="?",
      const=True,
      help="Ignore eventual 404",
    )
    parser.add_argument(
      "--fix-404-resources",
      default=False,
      nargs="?",
      const=True,
      help="Fix 404 resources",
    )
    parser.add_argument(
      "--fix-offline-servers",
      default=False,
      nargs="?",
      const=True,
      help="Remove resources from offline servers",
    )

  def handle(self, *args, **options):
    models = apps.get_models()

    ignored = set()
    if(options["ignore"]):
      for target in options["ignore"].split(","):
        ignored.add(urlparse(target).netloc)

    if(len(ignored) > 0):
      print("Ignoring servers:")
      for server in ignored:
        print("- "+server)

    resources = set()
    resources_map = dict()
    base_urls = set()

    for model in models:
      for obj in model.objects.all():
        if hasattr(obj, "urlid"):
          if(obj.urlid):
            if(not obj.urlid.startswith(settings.BASE_URL)):
              url = urlparse(obj.urlid).netloc
              if(url not in ignored):
                resources.add(obj.urlid)
                resources_map[obj.urlid] = obj
                base_urls.add(url)

    if(len(base_urls) > 0):
      print("Servers that I have backlinks to:")
      for server in base_urls:
        print("- "+server)
    else:
      print("I don't have any backlink")

    source_urls = set()
    for source in LDPSource.objects.all():
      source_urls.add(urlparse(source.urlid).netloc)

    if(len(source_urls) > 0):
      print("Servers that I'm allowed to get federated to:")
      for server in source_urls:
        print("- "+server)
    else:
      print("I'm not federated")

    difference_servers = base_urls.difference(source_urls)
    if(len(difference_servers) > 0):
      print("Servers that I should not get aware of:")
      for server in difference_servers:
        print("- "+server)

      if(not options["ignore_faulted"]):
        faulted_resources = set()
        for server in difference_servers:
          for resource in resources:
            if(urlparse(resource).netloc in server):
              faulted_resources.add(resource)

        if(len(faulted_resources) > 0):
          print("Resources in fault:")
          for resource in faulted_resources:
            print("- "+resource)
        else:
          print("No resource are in fault")

        if(options["fix_faulted_resources"]):
          for resource in faulted_resources:
            try:
              resources_map[resource].delete()
            except:
              pass
          print("Fixed faulted resources")
        else:
          print("Fix them with `./manage.py check_integrity --fix-faulted-resources`")
    else:
      print("I accept datas for every of those servers")

    if(not options["ignore_404"]):
      resources_404 = set()
      resources_servers_offline = set()
      for resource in resources:
        try:
          if(requests.get(resource).status_code == 404):
            resources_404.add(resource)
        except:
          resources_servers_offline.add(resource)

      if(len(resources_404) > 0):
        print("Faulted resources, 404:")
        for resource in resources_404:
          print("- "+resource)
        if(options["fix_404_resources"]):
          for resource in resources_404:
            try:
              resources_map[resource].delete()
            except:
              pass
          print("Fixed 404 resources")
        else:
          print("Fix them with `./manage.py check_integrity --fix-404-resources`")

      if(len(resources_servers_offline) > 0):
        print("Faulted resources, servers offline:")
        for resource in resources_servers_offline:
          print("- "+resource)
        if(options["fix_offline_servers"]):
          for resource in resources_servers_offline:
            try:
              resources_map[resource].delete()
            except:
              pass
          print("Fixed resources on offline servers")
        else:
          print("Fix them with `./manage.py check_integrity --fix-offline-servers`")

      else:
        print("No 404 in known resources")

    exit(0)
